﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Wordbee.Beebox.Extensibility;


namespace Wordbee.Beebox.Extensions.Samples.BadMarkupFilterPackage
{

	/// <summary>
	/// This is a simple Beebox extension that adds a text filter option to the user interface.
	/// </summary>
	public class BadMarkupTextFilter : BeeboxTextFilter
	{

		/// <summary>
		/// Gets the globally unique id of this extension.
		/// </summary>
		public override string ExtensionID { get { return "wb-filter-badmarkup"; } }


		/// <summary>
		/// Gets if the extension is by default enabled. If false, then it must be enabled explicitly in each
		/// Beebox project from the Beebox Admin UI.
		/// </summary>
		public override bool EnableByDefault { get { return true; } }



		/// <summary>
		/// Gets a name. Displayed in the Beebox extension manager page.
		/// </summary>
		public override string Name { get { return "Bad Markup Filter"; } }


		/// <summary>
		/// Gets the title of the algorithm shown in the user interface such as the "Actions" tool where a user can pseudo translate
		/// selected content.
		/// </summary>
		public override string UITitle { get { return "Bad Markup Filter"; } }


		/// <summary>
		/// Gets a description. Displayed in the Beebox extension manager page.
		/// </summary>
		public override string Description { get { return "This filter lets you find translations with incorrect markup."; } }


		/// <summary>
		/// Gets version number. Displayed in the Beebox extension manager page.
		/// </summary>
		public override string Version { get { return "1.0"; } }


		/// <summary>
		/// Gets the author. Displayed in the Beebox extension manager page.
		/// </summary>
		public override string Author { get { return "Wordbee"; } }




		/// <summary>
		/// Return true if the segment passes the filter. Otherwise do return false.
		/// </summary>
		/// <param name="segment">
		/// Segment details with source text, translation and additional properties:
		/// - Source: The original text
		/// - Target: The translated text (can be null or "" to indicate no translation yet)
		/// - SourceLocale: The source language code
		/// - TargetLocale: The target language code
		/// - Status: The Quality Assurance status: 0 = not set, 1 = QA Ok, 2 = QA Error
		/// - Validated: The translation currently is approved or not
		/// - Locked: The translation is locked for editing by the user
		/// - File: The file name containing the segment
		/// - Project: The project key
		/// - Key: Gets the segment key or the context of the segment. File format dependent: "Heading 1", "Hyperlink", ...
		/// - SourceNext: The text just before this segment
		/// - SourcePrevious: The text just after this segment
		/// </param>
		/// <param name="parameters">The extension's parameters. You need to implement the respective virtual methods to permit configuring the parameter individually per project.</param>
		/// <returns>
		/// True: The filter retains the segment
		/// False: The filter discards the segment
		/// </returns>
		public override bool IsMatch(Segment segment, IExtensionConfiguration parameters)
		{
			// If source has markup and target is empty then we yield this translation if source has markup
			var sourceHasMarkup = segment.Source.Contains(ExtensionsHelper.TAG_MARKER_OPEN);
			var targetHasMarkup = segment.Target?.Contains(ExtensionsHelper.TAG_MARKER_OPEN) ?? false;

			// Case no markup at all. We do not need to do more complex checks.
			if (!sourceHasMarkup && !targetHasMarkup)
				return false;

			// Case either one of the texts has markup
			if (sourceHasMarkup != targetHasMarkup)
				return true;

			// Create a dictionary with key = markup and value = count
			Dictionary<string, int> markups = new Dictionary<string, int>();

			// We add 1 for each markup in source
			AddMarkupToDictionary(segment.Source, markups, 1);

			// We add -1 for each markup in target
			AddMarkupToDictionary(segment.Target, markups, -1);

			// Any non-zero value points to a markup difference
			return markups.Any(m => m.Value != 0);
		}



		/// <summary>
		/// Add all markup to dictionary
		/// </summary>
		/// <param name="value">The name of markup</param>
		/// <param name="markups"></param>
		/// <param name="increment">Add this value to the markup</param>
		public void AddMarkupToDictionary(string value, Dictionary<string, int> markups, int increment)
		{
			int p = -1;
			while (true)
			{
				// Get next markup
				p = value.IndexOf(ExtensionsHelper.TAG_MARKER_OPEN, p + 1);
				if (p < 0) break;
				
				// Get end of markup name
				int pend = value.IndexOf(ExtensionsHelper.TAG_MARKER_CLOSE, p + 1);
				if (pend < 0) break;

				// Add to dictionary and increment/decrement value
				string name = value.Substring(p + 1, pend - p - 1);
				if (markups.ContainsKey(name))
					markups[name] += increment;
				else
					markups.Add(name, increment);

				p = pend;
			}
		}


	}
}
